/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
		       SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
		       INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
****************************************************************************/

#ifndef SMART_TOOLS_FILE_H
#define SMART_TOOLS_FILE_H

#include "smarttools.h"
#include "ststring.h"
#include "stpermission.h"
#include "stdebug.h"
#include "stassert.h"
#include "stoutput.h"

#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>

/*!
\brief This class allows for basic file operations.

With StFile you can open, close, read, write, move, copy, and remove a file.
It also allows operation on a file's permissions.
*/

class StFile
{
	public:
		StFile(){ init(); }
		StFile( const StString& path, bool followSymLinks=false );
		StFile( const char* path, bool followSymLinks=false ) { init(); setPath(path, followSymLinks); }
		StFile( const StFile& copy )
		{
			init();
			path = copy.path;
			_exists = copy._exists;
			st = copy.st;
		}
		~StFile() { ST_ASSERT( fPointer == NULL && "File should be closed before StFile destruction by close()." ); }
		
		/** Set path to the file. It can follow symlinks. */
		void setPath( const StString& path, bool followSymLinks=false )
		{ ST_DEBUG((StString)"StFile::setPath(\""+path+"\","+StString(followSymLinks)+")"); this->path = path; readStat( followSymLinks ); }
		
		/** \return path to the file. */
		StString getPath() const { return path; }
		
		/** \return base name of file. */
		StString getBaseName() const;
		
		/** \return directory path which this file is */
		StString getDirName() const;
		
		/** \return true or false depeding on if the file exists or not.*/
		bool exists() const { return _exists; }
		
		/** Read current file state */
		void readStat( bool followSymLinks=true );
		
		// stats
		bool isDirectory() const { return S_ISDIR( st.st_mode ); }
		bool isSymLink() const { return S_ISLNK( st.st_mode ); }
		bool isRegular() const { return S_ISREG( st.st_mode ); }
		bool isCharacterDevice() const { return S_ISCHR( st.st_mode ); }
		bool isBlockDevice() const { return S_ISBLK( st.st_mode ); }
		bool isFifo() const { return S_ISFIFO( st.st_mode ); }
		bool isSocket() const { return S_ISSOCK( st.st_mode ); }
		
		/** \return file's size. */
		long size() const { return st.st_size; }
		
		/** Remove a file. */
		bool remove();
		
		/** Move file to the dest and change current path if succeed. */
		bool move( StString dest, bool overwrite=false );
		
		/** Copy file to the dest. */
		bool copy( StString dest, bool overwrite=false );
		
		/** \return all permisions for this file. */
		StPermission getPermissions() const;
		
		/** Change file's permissions */
		bool changePermissions( const StPermission& perm );
		
		/** Change owner and/or group if given. */
		bool changeOwner( const StPermission& perm );
		
		/** Open file. */
		bool open( SmartTools::OpenFileMode mode );
		
		/** Open file. */
		bool open( const StString& mode );
		
		/** Close file. */
		void close(){  ST_ASSERT(fPointer != NULL && "File not open"); fclose(fPointer); fPointer=NULL; }
		
		/**
		Read from file to the buffer.
		\param *buffer Pointer to the destination structure with a minimum size of (size*count) bytes.
		\param size Size in bytes for each item to be read.
		\param count Number of items, each one with a size in bytes.
		\warning File must be open before this.
		\return The total number of items read is returned.
		*/
		unsigned read( void *buffer, unsigned count, unsigned size=1 )
		{
			ST_ASSERT(fPointer != NULL && "File not open");
			return fread( buffer, size, count, fPointer );
		}
		
		/**
		Write from buffer to the file.
		\param *buffer Pointer to data to be written.
		\param size Size in bytes of each item that has to be written.
		\param count Number of items, each one with a size in bytes.
		\warning File must be open before this.
		\return Number of full items (not bytes) successfully written.
		*/
		unsigned write( const void *buffer, unsigned size, unsigned count=1 )
		{
			ST_ASSERT(fPointer != NULL && "File not open");
			return fwrite( buffer, size, count, fPointer );
		}
		
		/**
		\return line from file.
		\warning File must be opened first by open().
		*/
		StString getLine();
		
		bool isEndOfFile()
		{ ST_ASSERT(fPointer != NULL && "File not open"); return ( feof(fPointer) == 0 ) ? false : true; }
		
		/** Set the path to the file. */
		void operator=( const StString& path ){ setPath(path); }
		
		/** Returns full path of file. */
		operator StString() const { return getPath(); }
		
		/**
		Write string to file.
		\warning File must be opened first by open().
		*/
		StFile& operator<<( const StString& string ){
			ST_ASSERT( fPointer != NULL && "File must be open by open() function" );
			write( string.c_str(), string.length() );
			return *this;
		}
		
		/** Clear path and file's pointer. */
		void clear();
	
	private:
		StString path;
		
		bool _exists;
		struct stat st;
		FILE* fPointer;
		
		void init();
};

#endif //SMART_TOOLS_FILE_H
